/*
 * main.c
 *
 *  Created on: 12 sierpien 2016
 *      Author: Przemek
 */

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include <math.h>
#include "nokia5110lcd.h"
#include "przyciski.h"
#include "snake.h"

volatile uint8_t timer0_50Hz;

// zmienna wprowadzajaja element losowosci
uint8_t ziarno = 0;

// pauza/gra
uint8_t pauza = 0;

// stan animacji
uint8_t stan_animacji = 0;

// jesli wynosi 0 - gra jest na ekranie poczatkowym
uint8_t stan_gry = 0;

// konfiguracja timera odmierzajacego czas na 50Hz dla kwarcu 8MHz
void timer0_50hz_config( void );

void obsluga_przyciskow_w_grze( void );

int main(void)
{
	_delay_ms(1000);
	timer0_50hz_config(); 	// inicjacja timera na ktorym zrobione jest opoznienie
	przyciski_init();		// inicjalizacja przyciskow
	nokialcd_init();		// inicjalizacja wyswietlacza
	stan_gry = 0;
	stan_animacji = 0;
	pauza = 0;
	sei();					// uruchomienie przerwan ( poniewaz odmierzanie czasu jest na timerze )
	while (1)
	{
		ziarno++;
		obsluga_przyciskow_w_grze();

		// odswiezenie ekranu 5 razy na sekunde
		// za pomoca ustawien timera, lub wartosci, ktora jest wpisywana do zmiennej
		// "timer0_50Hz" mozesz sterowac szybkoscia gry ( trudnoscia )
		if(!timer0_50Hz)
		{
			if( stan_gry == 0 )
			{
				// obsluga animacji startowej
				snake_animacja( stan_animacji );
				stan_animacji++;
				if(stan_animacji == 9 )
				{
					stan_animacji = 1;
				}
			}
			else
			{
				if( !pauza )
				{
					// wykonanie ruchu w grze
					if( snake_ruch() == 0 )
					{

					}
					else
					{
						stan_gry = 0;
						stan_animacji = 0;
					}
				}
				// odswiezenie ekranu
				nokialcd_buf_wyswietl();
			}

			// wpisanie wartosci do zmiennej odmierzajacej czas (domyslnie 10)
			timer0_50Hz = 10;
		}
	}
}

ISR(TIMER0_OVF_vect)
{
	// wartosc wyliczona dla ok 50Hz
	TCNT0 = 100;

	// dekrementacja zmiennej
	if(timer0_50Hz)
	{
		timer0_50Hz--;
	}
}

// konfiguracja timera odmierzajacego czas na 50Hz dla kwarcu 8MHz
void timer0_50hz_config(void)
{
	TCCR0 |= (1 << CS00) | (1 << CS02); // presk 1024
	TIMSK |= ( 1 << TOIE0);//zezwolenie na przerwanie
}

void obsluga_przyciskow_w_grze()
{
	// ruch w lewo
	if( przycisk1_obsluga() )
	{
		snake_kierunek_lewo();
		ziarno++;
	}

	// ruch w gore
	if( przycisk2_obsluga() )
	{
		snake_kierunek_gora();
		ziarno++;
	}

	// start/pauza
	if(przycisk3_obsluga())
	{
		if( stan_gry == 0 )
		{
			pauza = 0;
			stan_gry = 1;
			snake_nowa_gra();
		}
		else
		{
			pauza = pauza ? 0 : 1;
		}
		ziarno++;
	}

	// ruch na dol
	if(przycisk4_obsluga() )
	{
		snake_kierunek_dol();
		ziarno++;
	}

	// ruch w prawo
	if(przycisk5_obsluga() )
	{
		snake_kierunek_prawo();
		ziarno++;
	}
}


