/*
 * snake.c
 *
 *  Created on: 13 sie 2016
 *      Author: Przemek
 */

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include <math.h>
#include "nokia5110lcd.h"
#include "przyciski.h"
#include "snake.h"

const char const s1[] PROGMEM = {
#include "s1.h"
};

const char const s2[] PROGMEM = {
#include "s2.h"
};

const char const s3[] PROGMEM = {
#include "s3.h"
};

const char const s4[] PROGMEM = {
#include "s4.h"
};

const char const s5[] PROGMEM = {
#include "s5.h"
};


const char const napis[] PROGMEM = {
#include "napis.h"
};



enum {gora,dol,lewo,prawo};
uint8_t snake_kolejka[112];

uint8_t snake_glowa_index = 2;
uint8_t snake_ogon_index = 1;
uint8_t snake_kierunek = prawo;
uint8_t owoc_pozycja;

// zmienna obslugujaca pseudolosowosc w grze, inkrementowac w roznych miejscach w mainie
extern uint8_t ziarno;

// rysuje owoc w danej pozycji
void snake_narysuj_owoc(uint8_t numer)
{
	nokialcd_buf_rysuj_prostokat((numer%14)* SZEROKOSC_SEGMENTU,(numer/14)* 6,SZEROKOSC_SEGMENTU,SZEROKOSC_SEGMENTU,1);
	nokialcd_buf_rysuj_prostokat((numer%14)* SZEROKOSC_SEGMENTU +1,(numer/14)* 6+1,SZEROKOSC_SEGMENTU-2,SZEROKOSC_SEGMENTU-2,1);
}

// rysuje/zamazuje owoc w wybranej pozycji
void snake_narysuj_owoc2(uint8_t numer,uint8_t kolor)
{
	uint8_t tempx = (numer%ILOSC_SEGMENTOW_X) * SZEROKOSC_SEGMENTU + 1;
	uint8_t tempy = (numer/ILOSC_SEGMENTOW_X) * SZEROKOSC_SEGMENTU + 1;
	nokialcd_buf_zapal_pixel(tempx,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 3,tempy,kolor);
	tempy++;
	nokialcd_buf_zapal_pixel(tempx + 1,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 2,tempy,kolor);
	tempy++;
	nokialcd_buf_zapal_pixel(tempx + 1,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 2,tempy,kolor);
	tempy++;
	nokialcd_buf_zapal_pixel(tempx,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 3,tempy,kolor);
}


// rysuje/zmazuje segment naszego weza na okreslonej pozycji
void snake_narysuj_segment(uint8_t numer,uint8_t kolor)
{
	uint8_t tempx = (numer%ILOSC_SEGMENTOW_X) * SZEROKOSC_SEGMENTU;
	uint8_t tempy = (numer/ILOSC_SEGMENTOW_X) * SZEROKOSC_SEGMENTU;

	nokialcd_buf_zapal_pixel(tempx + 2,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 3,tempy,kolor);
	tempy++;
	nokialcd_buf_zapal_pixel(tempx + 1,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 2,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 3,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 4,tempy,kolor);
	tempy++;
	nokialcd_buf_zapal_pixel(tempx,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 1,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 4,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 5,tempy,kolor);
	tempy++;
	nokialcd_buf_zapal_pixel(tempx,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 1,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 4,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 5,tempy,kolor);
	tempy++;
	nokialcd_buf_zapal_pixel(tempx + 1,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 2,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 3,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 4,tempy,kolor);
	tempy++;
	nokialcd_buf_zapal_pixel(tempx + 2,tempy,kolor);
	nokialcd_buf_zapal_pixel(tempx + 3,tempy,kolor);
}

// kasuje ostatni segment naszego weza
void snake_skasuj_ostatni()
{
	snake_kolejka[ snake_ogon_index ] = 99;
	snake_ogon_index = (snake_ogon_index + 1) % ROZMIAR_KOLEJKI;
}

// zapisuje nowa glowe weza z dana pozycja
void snake_zapisz(uint8_t dana)
{
	snake_glowa_index = (snake_glowa_index+1)%ROZMIAR_KOLEJKI;
	snake_kolejka[snake_glowa_index] = dana;
}

//zwraca 1 gdy pole jest puste, a 0 gdy jest zajete przez weza lub 2 gdy jest tam owoc
uint8_t snake_sprawdz_kolizje(uint8_t pozycja)
{
	if(snake_glowa_index != snake_ogon_index)
	{
		uint8_t temp = (snake_glowa_index+1)%ROZMIAR_KOLEJKI;
		for(uint8_t i = snake_ogon_index; i != temp; i = (i+1)%ROZMIAR_KOLEJKI)
		{
			if(snake_kolejka[i] == pozycja)
			{
				return 0;
			}
			if(snake_kolejka[i] == owoc_pozycja)
			{
				return 2;
			}
		}
		return 1;
	}
	else
	{
		if(snake_kolejka[snake_ogon_index] == owoc_pozycja)
		{
			return 2;
		}
	}
	return 1;
}

// sprawdza kolizje glowy weza z jego cialem
uint8_t snake_sprawdz_kolizje2(uint8_t pozycja)
{
	uint8_t temp = (snake_glowa_index+1)%ROZMIAR_KOLEJKI;
	for(uint8_t i = snake_ogon_index; i != temp; i = (i+1)%ROZMIAR_KOLEJKI)
	{
		if(snake_kolejka[i] == pozycja)
		{
			return 0;
		}
	}
	return 1;
}

// dodaje na planszy nowy owoc do zebrania
void snake_dodaj_owoc(void)
{
	uint8_t pozycja_temp = ziarno%ROZMIAR_KOLEJKI;
	uint8_t temp = snake_sprawdz_kolizje(pozycja_temp);
	while(temp != 1)
	{
		pozycja_temp = (pozycja_temp + 1)%ROZMIAR_KOLEJKI;
		temp = snake_sprawdz_kolizje2(pozycja_temp);
	}
	owoc_pozycja = pozycja_temp;
	snake_narysuj_owoc2(owoc_pozycja,1);
}

// pobiera ostatni element naszego weza
uint8_t snake_pobierz_ostatni(void)
{
	uint8_t temp = snake_kolejka[snake_ogon_index];
	snake_ogon_index = (snake_ogon_index+1)%ROZMIAR_KOLEJKI;
	return temp;
}

// zwraca dlugosc naszego weza
uint8_t snake_dlugosc(void)
{
	if(snake_glowa_index > snake_ogon_index)
	{
		return (snake_glowa_index - snake_ogon_index + 1);
	}
	else
	{
		return (ROZMIAR_KOLEJKI + snake_glowa_index - snake_ogon_index + 1);
	}
}

// czysci bufor weza
void snake_czysc_kolejke(void)
{
	for(uint8_t i = 0; i < ROZMIAR_KOLEJKI; i++)
	{
		snake_kolejka[i] = 99;
	}
}

// rozpoczyna nowa gre - czysci bufory, przyznaje poczatkowe wartosci zmiennych
void snake_nowa_gra(void)
{
	nokialcd_czysc_ekran();
	snake_czysc_kolejke();
	snake_kolejka[2] = 15;
	snake_glowa_index = 2;
	snake_ogon_index = 2;
	snake_kierunek = prawo;
	snake_dodaj_owoc();
}

void snake_animacja( uint8_t klatka )
{
	switch( klatka )
	{
	case 0:
	{
		nokialcd_buf_czysc_bufor();
		nokialcd_czysc_ekran();
		nokialcd_wyswietl_obrazek(napis,5,0,72,1);
		break;
	}
	case 1:
	{
		nokialcd_wyswietl_obrazek(s1,16,2,54,4);
		break;
	}
	case 2:
	{
		nokialcd_wyswietl_obrazek(s2,16,2,54,4);
		break;
	}
	case 3:
	{
		nokialcd_wyswietl_obrazek(s3,16,2,54,4);
		break;
	}
	case 4:
	{
		nokialcd_wyswietl_obrazek(s4,16,2,54,4);
		break;
	}
	case 5:
	{
		nokialcd_wyswietl_obrazek(s5,16,2,54,4);
		break;
	}
	case 6:
	{
		nokialcd_wyswietl_obrazek(s4,16,2,54,4);
		break;
	}
	case 7:
	{
		nokialcd_wyswietl_obrazek(s3,16,2,54,4);
		break;
	}
	case 8:
	{
		nokialcd_wyswietl_obrazek(s2,16,2,54,4);
		break;
	}
	}

}

// funkcja uruchamiana w przypadku przegranej
void snake_przegrana(void)
{
// miganie ekranem a potem nowa gra
	nokialcd_buf_wyswietl();
	_delay_ms(400);
	nokialcd_czysc_ekran();
	_delay_ms(400);
	nokialcd_buf_wyswietl();
	_delay_ms(400);
	nokialcd_czysc_ekran();
	_delay_ms(400);
	nokialcd_buf_wyswietl();
	_delay_ms(400);
	nokialcd_czysc_ekran();
	_delay_ms(400);
	nokialcd_buf_czysc_bufor();
	snake_nowa_gra();
}

void snake_kierunek_lewo()
{
	if(snake_kierunek != prawo)
	{
		snake_kierunek = lewo;
	}
}

void snake_kierunek_prawo()
{
	if(snake_kierunek != lewo)
	{
		snake_kierunek = prawo;
	}
}

void snake_kierunek_gora()
{
	if(snake_kierunek != dol)
	{
		snake_kierunek = gora;
	}
}

void snake_kierunek_dol()
{
	if(snake_kierunek != gora)
	{
		snake_kierunek = dol;
	}
}


// powoduje przesuniecie weza o 1 jednostke
// zwraca 0 jesli ruch byl ok, 1 jesli przegrano gre
uint8_t snake_ruch()
{
	uint8_t nastepna_pozycja = 0;
	// wyznaczenie nowej pozycji dla glowy w zaleznosci od kierunku poruszania
	if(snake_kierunek == prawo)
	{
		if(snake_kolejka[snake_glowa_index] % ILOSC_SEGMENTOW_X == (ILOSC_SEGMENTOW_X - 1))
		{
			nastepna_pozycja = snake_kolejka[snake_glowa_index] - (ILOSC_SEGMENTOW_X - 1);
		}
		else
		{
			nastepna_pozycja = snake_kolejka[snake_glowa_index] + 1;
		}
	}

	else if(snake_kierunek == lewo)
	{
		if(snake_kolejka[snake_glowa_index]%ILOSC_SEGMENTOW_X == 0)
		{
			nastepna_pozycja = snake_kolejka[snake_glowa_index] + ILOSC_SEGMENTOW_X - 1;
		}
		else
		{
			nastepna_pozycja = snake_kolejka[snake_glowa_index] - 1;
		}
	}

	else if(snake_kierunek == gora)
	{
		if(snake_kolejka[snake_glowa_index]/ILOSC_SEGMENTOW_X == 0)
		{
			nastepna_pozycja = snake_kolejka[snake_glowa_index] + ROZMIAR_KOLEJKI - ILOSC_SEGMENTOW_X;
		}
		else
		{
			nastepna_pozycja = snake_kolejka[snake_glowa_index] - ILOSC_SEGMENTOW_X;
		}
	}

	else if(snake_kierunek == dol)
	{
		if(snake_kolejka[snake_glowa_index] > ROZMIAR_KOLEJKI - ILOSC_SEGMENTOW_X)
		{
			nastepna_pozycja = snake_kolejka[snake_glowa_index]%ILOSC_SEGMENTOW_X;
		}
		else
		{
			nastepna_pozycja = snake_kolejka[snake_glowa_index] + ILOSC_SEGMENTOW_X;
		}
	}

	// sprawdzenie czy nastepne pole jest wolne
	uint8_t temp = snake_sprawdz_kolizje(nastepna_pozycja);

	if( temp == 1)//  pole jest wolne wiec narysuj glowe i zmaz ogon
	{
		// dodaj segment glowy do kolejki
		snake_zapisz(nastepna_pozycja);

		// narysuj nowy segment
		snake_narysuj_segment(nastepna_pozycja,1);

		// zmazujemy segment ogona
		snake_narysuj_segment(snake_kolejka[snake_ogon_index],0);

		// kasujemy ostatni element kolejki
		snake_skasuj_ostatni();
	}
	else if (temp == 2) // zderzenie z owocem - rosniecie weza + nowy owoc
	{
		// dodajemy nowy owoc
		snake_dodaj_owoc();

		// rysujemy owoc
		snake_narysuj_owoc2(snake_kolejka[snake_glowa_index],0);

		// przesunmiecie weza
		snake_zapisz(nastepna_pozycja);
		snake_narysuj_segment(nastepna_pozycja,1);
	}
	else // zderzenie z jakas czescia - przegrana
	{
		snake_przegrana();
		return 1;
	}
	return 0;
}
